import { isBuiltin } from 'node:module';
import { IGNORE_DEFINITELY_TYPED, IGNORED_DEPENDENCIES, IGNORED_GLOBAL_BINARIES } from './constants.js';
import { isDefinitelyTyped, getDefinitelyTypedFor, getPackageFromDefinitelyTyped } from './util/modules.js';
export class DependencyDeputy {
    isStrict;
    _manifests = new Map();
    referencedDependencies;
    peerDependencies;
    installedBinaries;
    constructor({ isStrict }) {
        this.isStrict = isStrict;
        this.referencedDependencies = new Map();
        this.peerDependencies = new Map();
        this.installedBinaries = new Map();
    }
    addWorkspace({ name, dir, manifestPath, manifest, ignoreDependencies, ignoreBinaries, }) {
        const scripts = Object.values(manifest.scripts ?? {});
        const dependencies = Object.keys(manifest.dependencies ?? {});
        const peerDependencies = Object.keys(manifest.peerDependencies ?? {});
        const optionalDependencies = Object.keys(manifest.optionalDependencies ?? {});
        const devDependencies = Object.keys(manifest.devDependencies ?? {});
        const allDependencies = [...dependencies, ...devDependencies, ...peerDependencies, ...optionalDependencies];
        this._manifests.set(name, {
            workspaceDir: dir,
            manifestPath,
            ignoreDependencies,
            ignoreBinaries,
            scripts,
            dependencies,
            peerDependencies,
            optionalDependencies,
            devDependencies,
            allDependencies,
        });
    }
    getWorkspaceManifest(workspaceName) {
        return this._manifests.get(workspaceName);
    }
    getProductionDependencies(workspaceName) {
        const manifest = this._manifests.get(workspaceName);
        if (!manifest)
            return [];
        if (this.isStrict)
            return [...manifest.dependencies, ...manifest.peerDependencies];
        return manifest.dependencies;
    }
    getDevDependencies(workspaceName) {
        return this._manifests.get(workspaceName)?.devDependencies ?? [];
    }
    setInstalledBinaries(workspaceName, installedBinaries) {
        this.installedBinaries.set(workspaceName, installedBinaries);
    }
    getInstalledBinaries(workspaceName) {
        return this.installedBinaries.get(workspaceName);
    }
    addReferencedDependency(workspaceName, packageName) {
        if (!this.referencedDependencies.has(workspaceName)) {
            this.referencedDependencies.set(workspaceName, new Set());
        }
        this.referencedDependencies.get(workspaceName)?.add(packageName);
    }
    addPeerDependencies(workspaceName, peerDependencies) {
        this.peerDependencies.set(workspaceName, peerDependencies);
    }
    getPeerDependencies(workspaceName, dependency) {
        return Array.from(this.peerDependencies.get(workspaceName)?.get(dependency) ?? []);
    }
    maybeAddReferencedExternalDependency(workspace, packageName) {
        if (isBuiltin(packageName))
            return true;
        if (packageName === workspace.pkgName)
            return true;
        const workspaceNames = this.isStrict ? [workspace.name] : [workspace.name, ...[...workspace.ancestors].reverse()];
        const closestWorkspaceName = workspaceNames.find(name => this.isInDependencies(name, packageName));
        if (this.getWorkspaceManifest(workspace.name)?.ignoreDependencies.includes(packageName))
            return true;
        const typesPackageName = !isDefinitelyTyped(packageName) && getDefinitelyTypedFor(packageName);
        const closestWorkspaceNameForTypes = typesPackageName && workspaceNames.find(name => this.isInDependencies(name, typesPackageName));
        if (closestWorkspaceName || closestWorkspaceNameForTypes) {
            closestWorkspaceName && this.addReferencedDependency(closestWorkspaceName, packageName);
            closestWorkspaceNameForTypes && this.addReferencedDependency(closestWorkspaceNameForTypes, typesPackageName);
            return true;
        }
        if (IGNORED_GLOBAL_BINARIES.includes(packageName))
            return true;
        if (this.getWorkspaceManifest(workspace.name)?.ignoreBinaries.includes(packageName))
            return true;
        for (const name of workspaceNames) {
            const binaries = this.getInstalledBinaries(name);
            if (binaries?.has(packageName)) {
                const dependencies = binaries.get(packageName);
                if (dependencies?.size) {
                    dependencies.forEach(dependency => this.addReferencedDependency(name, dependency));
                    return true;
                }
            }
        }
        return false;
    }
    isInDependencies(workspaceName, packageName) {
        const manifest = this._manifests.get(workspaceName);
        if (!manifest)
            return false;
        const dependencies = this.isStrict ? this.getProductionDependencies(workspaceName) : manifest.allDependencies;
        return dependencies.includes(packageName);
    }
    settleDependencyIssues() {
        const dependencyIssues = [];
        const devDependencyIssues = [];
        for (const [workspaceName, { manifestPath, ignoreDependencies, ignoreBinaries }] of this._manifests.entries()) {
            const referencedDependencies = this.referencedDependencies.get(workspaceName);
            const installedBinaries = this.getInstalledBinaries(workspaceName);
            const ignoreBins = [...IGNORED_GLOBAL_BINARIES, ...ignoreBinaries];
            const ignoreDeps = [...IGNORED_DEPENDENCIES, ...ignoreDependencies];
            const isNotIgnoredDependency = (packageName) => !ignoreDeps.includes(packageName);
            const isNotIgnoredBinary = (packageName) => {
                if (installedBinaries?.has(packageName)) {
                    const binaryNames = installedBinaries.get(packageName);
                    if (binaryNames && ignoreBins.some(ignoredBinary => binaryNames.has(ignoredBinary)))
                        return false;
                }
                return true;
            };
            const isNotReferencedDependency = (dependency) => {
                if (referencedDependencies?.has(dependency))
                    return false;
                const [scope, typedDependency] = dependency.split('/');
                if (scope === '@types') {
                    const typedPackageName = getPackageFromDefinitelyTyped(typedDependency);
                    if (IGNORE_DEFINITELY_TYPED.includes(typedPackageName))
                        return false;
                    const peerDependencies = this.getPeerDependencies(workspaceName, typedPackageName);
                    if (peerDependencies.length) {
                        return !peerDependencies.find(peerDependency => !isNotReferencedDependency(peerDependency));
                    }
                    return !referencedDependencies?.has(typedPackageName);
                }
                if (!referencedDependencies?.has(dependency)) {
                    const peerDependencies = this.getPeerDependencies(workspaceName, dependency);
                    return !peerDependencies.find(peerDependency => !isNotReferencedDependency(peerDependency));
                }
                return false;
            };
            this.getProductionDependencies(workspaceName)
                .filter(isNotIgnoredDependency)
                .filter(isNotIgnoredBinary)
                .filter(isNotReferencedDependency)
                .forEach(symbol => dependencyIssues.push({ type: 'dependencies', filePath: manifestPath, symbol }));
            this.getDevDependencies(workspaceName)
                .filter(isNotIgnoredDependency)
                .filter(isNotIgnoredBinary)
                .filter(isNotReferencedDependency)
                .forEach(symbol => devDependencyIssues.push({ type: 'devDependencies', filePath: manifestPath, symbol }));
        }
        return { dependencyIssues, devDependencyIssues };
    }
}
