import { compact } from '../../util/array.js';
import { isAbsolute, isInternal, join, dirname } from '../../util/path.js';
import { load } from '../../util/plugin.js';
import { _resolve } from '../../util/require.js';
import { fallback } from './fallback.js';
const getDependencies = (config) => {
    const extendsSpecifiers = config.extends ? [config.extends].flat().map(resolveExtendsSpecifier) : [];
    if (extendsSpecifiers.includes('eslint-plugin-prettier'))
        extendsSpecifiers.push('eslint-config-prettier');
    const plugins = config.plugins ? config.plugins.map(resolvePluginPackageName) : [];
    const parser = config.parser;
    const extraParsers = config.parserOptions?.babelOptions?.presets ?? [];
    const settings = config.settings ? getDependenciesFromSettings(config.settings) : [];
    return compact([...extendsSpecifiers, ...plugins, parser, ...extraParsers, ...settings]);
};
export const getDependenciesDeep = async (configFilePath, dependencies = new Set(), options) => {
    const addAll = (deps) => deps.forEach(dependency => dependencies.add(dependency));
    let config = configFilePath.endsWith('package.json') ? options.manifest.eslintConfig : undefined;
    if (!config) {
        try {
            config = await load(configFilePath);
        }
        catch (err) {
            if (err instanceof Error && err.cause instanceof Error && /Failed to patch ESLint/.test(err.cause.message)) {
                const dependencies = await fallback(configFilePath, options);
                addAll(dependencies);
            }
            else {
                throw err;
            }
        }
    }
    if (config) {
        if (config.extends) {
            for (const extend of [config.extends].flat()) {
                if (isInternal(extend)) {
                    const filePath = isAbsolute(extend) ? extend : join(dirname(configFilePath), extend);
                    const extendConfigFilePath = _resolve(filePath);
                    dependencies.add(extendConfigFilePath);
                    addAll(await getDependenciesDeep(extendConfigFilePath, dependencies, options));
                }
            }
        }
        if (config.overrides)
            for (const override of [config.overrides].flat())
                addAll(getDependencies(override));
        addAll(getDependencies(config));
    }
    return dependencies;
};
const resolvePackageName = (namespace, pluginName) => {
    return pluginName.startsWith('@')
        ? pluginName.includes('/')
            ? pluginName.replace(/\//, `/${namespace}-`)
            : `${pluginName}/${namespace}`
        : `${namespace}-${pluginName}`;
};
export const resolvePluginPackageName = (pluginName) => resolvePackageName('eslint-plugin', pluginName);
const resolveExtendsSpecifier = (specifier) => {
    if (isInternal(specifier))
        return;
    if (specifier.includes(':')) {
        const pluginName = specifier.replace(/^plugin:/, '').replace(/(\/|:).+$/, '');
        if (pluginName === 'eslint')
            return;
        return resolvePackageName('eslint-plugin', pluginName);
    }
    return specifier.includes('eslint') ? specifier : resolvePackageName('eslint-config', specifier);
};
const getImportPluginDependencies = (settings) => {
    const knownKeys = ['typescript'];
    if (Array.isArray(settings))
        return [];
    return Object.keys(settings)
        .filter(key => key !== 'node')
        .map(key => (knownKeys.includes(key) ? `eslint-import-resolver-${key}` : key));
};
export const getDependenciesFromSettings = (settings = {}) => {
    return compact(Object.entries(settings).reduce((packageNames, [settingKey, settings]) => {
        if (/^import\/(parsers|resolvers)?/.test(settingKey)) {
            return [...packageNames, ...getImportPluginDependencies(settings)];
        }
        return packageNames;
    }, []));
};
